/*******************************************************************************
 * Copyright (c) 2005 BEA Systems, Inc. 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    mkaufman@bea.com - initial API and implementation
 *    
 *******************************************************************************/

package demo;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import com.sun.mirror.apt.AnnotationProcessor;
import com.sun.mirror.apt.AnnotationProcessorEnvironment;
import com.sun.mirror.apt.Messager;
import com.sun.mirror.declaration.AnnotationMirror;
import com.sun.mirror.declaration.AnnotationTypeDeclaration;
import com.sun.mirror.declaration.AnnotationTypeElementDeclaration;
import com.sun.mirror.declaration.AnnotationValue;
import com.sun.mirror.declaration.Declaration;

public class DemoAnnotationProcessor implements AnnotationProcessor
{

	public DemoAnnotationProcessor(AnnotationProcessorEnvironment env)
	{
		_env = env;
	}

	public void process()
	{
		Messager messager = _env.getMessager();
		
		// obtain the declaration of the annotation we want to process
		AnnotationTypeDeclaration annoDecl = (AnnotationTypeDeclaration)_env.getTypeDeclaration(DemoAnnotation.class.getName());
		
		// get the annotated types
		Collection<Declaration> annotatedTypes = _env.getDeclarationsAnnotatedWith(annoDecl);
		
		for (Declaration decl : annotatedTypes) {
			Collection<AnnotationMirror> mirrors = decl.getAnnotationMirrors();
			
			// for each annotation found, get a map of element name/value pairs
			for (AnnotationMirror mirror : mirrors) {
				Map<AnnotationTypeElementDeclaration, AnnotationValue> valueMap = mirror.getElementValues();
				Set<Map.Entry<AnnotationTypeElementDeclaration, AnnotationValue>> valueSet = valueMap.entrySet();
				
				// the annotation processor understands two elements: "what" and "howMany"
				for (Map.Entry<AnnotationTypeElementDeclaration, AnnotationValue> annoKeyValue : valueSet) {					
					AnnotationValue annoValue = annoKeyValue.getValue();	// get the name
					if (annoKeyValue.getKey().getSimpleName().equals("what")) {
						Object whatValue = annoValue.getValue();			// get the value
						if (whatValue instanceof String) {
							if (!((String)whatValue).equals("fish")) {		// complain about invalid value
								messager.printError(annoValue.getPosition(), "But I don't like " + whatValue);
							}
						}
					}
					else if (annoKeyValue.getKey().getSimpleName().equals("howMany")) {
						Object howManyValue = annoValue.getValue();
						if (howManyValue instanceof Integer) {
							if (((Integer)howManyValue) > 6) {
								messager.printError(annoValue.getPosition(), howManyValue + " is too many");
							}
						}
					}
				}
			}
		}
	}

	public AnnotationProcessorEnvironment getEnvironment()
	{
		return _env;
	}

	AnnotationProcessorEnvironment	_env;

}
